#include <cstdio>
#include <cstring>


#include "error.h"
#include "cloud.h"
#include "TriMesh.h"
#include "trimeshutil.h"
#include "pointsample.h"
#include "stratify.h"

TriMesh *mesh;
Cloud cloud;
Array<char> mask;

int main(int argc, char *argv[])
{
    srand48(clock() + time(NULL));
    if (argc < 5) {
        fprintf(stderr, "Usage\n"
			  "sample-cmd <level> <lambda> <distance> <ply>\n");
        return 1;
    }
    debugf(("Loading mesh"));
    mesh = TriMesh::read(argv[4]);
    if (!mesh) {
        errorf(("Unable to load mesh '%s'", argv[4]));
        return 1;
    }
    debugf(("%d vertices, %d faces", mesh->vertices.size(), 
                mesh->faces.size()));
    debugf(("Generating stratified distribution"));
	int level = atoi(argv[1]);
	double lambda = strtod(argv[2], NULL);
	double distance = strtod(argv[3], NULL);
    StratifiedSample(mesh, level, lambda, distance, &cloud, mask);
	FILE *fout = fopen("stratified.pts", "wb");
    int nsamples = 0;
#if 0
	for (int i = 0; i < cloud.vertex.GetSize(); i++) {
        if (mask[i]) {
            if (cloud.vertex[i][2] > 0) {
                fprintf(fout, "%g %g\n", cloud.vertex[i][0], cloud.vertex[i][1]); 
                nsamples++;
            }
		}
    }
#endif
#if 1
    float buf[3*1024];
    int nbuf = 0;
	for (int i = 0; i < cloud.vertex.GetSize(); i++) {
        if (mask[i]) {
            buf[nbuf++] = (float) cloud.vertex[i][0];
            buf[nbuf++] = (float) cloud.vertex[i][1];
            buf[nbuf++] = (float) cloud.vertex[i][2];
            if (nbuf >= 3*1024) {
                fwrite(buf, 3*1024, sizeof(float), fout); 
                nbuf = 0;
            }
            nsamples++;
		}
    }
    fwrite(buf, nbuf, sizeof(float), fout); 
#endif
	fclose(fout);
    debugf(("Wrote 'stratified.bpts' with %d points", nsamples));
#if 0
    debugf(("Generating associated uniform distribution"));
    PointSample(mesh, nsamples, &cloud);
    fout = fopen("uniform.pts", "wb");
	for (int i = 0; i < cloud.vertex.GetSize(); i++) {
        fprintf(fout, "%g %g\n", cloud.vertex[i][0], cloud.vertex[i][1]); 
    }
#if 0
    nbuf = 0;
	for (int i = 0; i < cloud.vertex.GetSize(); i++) {
        buf[nbuf++] = (float) cloud.vertex[i][0];
        buf[nbuf++] = (float) cloud.vertex[i][1];
        buf[nbuf++] = (float) cloud.vertex[i][2];
        if (nbuf >= 3*1024) {
            fwrite(buf, 3*1024, sizeof(float), fout); 
            nbuf = 0;
        }
    }
    fwrite(buf, nbuf, sizeof(float), fout); 
#endif
	fclose(fout);
    debugf(("Wrote 'uniform.bpts'"));
#endif
    return 0;
}
