#ifndef SAMPLEWINDOW_H
#define SAMPLEWINDOW_H

#include "trimeshwindow.h"
#include "cloud.h"
#include "stratify.h"

class SampleWindow: public TriMeshWindow {
public:
    SampleWindow(); 
    virtual ~SampleWindow(); 
    void SetCloud(Cloud *cloud); 
    void SetOctree(SamplerOctree *tree); 
    void SetMask(Array<char> *mask); 
    Cloud *GetCloud(void) const;
    SamplerOctree *GetOctree(void) const;

    void Save(void);
    void Load(void);
    void DrawModel(void); 
    void DrawSample(const Vector3 &p);
    void SetSampleSize(double p, double s); 

    void Button(int button, int state, int x, int y);
    void Motion(int x, int y);
    void Keyboard(unsigned char c, int x, int y);
private:
    void DrawOctree(void); 
    void DrawSamples(void); 
    void DrawSelected(void); 
    void DrawNeighbors(void); 
    void DrawNormals(void); 
    void DrawOriginal(void); 
    void UpdatePick(int x, int y);
    void ColorSamples(void);
    double GetPointSize(void);
    double GetSphereSize(void);
    int PickByColor(int mx, int my);
    Cloud *cloud;
    int color_list;
    int selected;
    bool drag;
    double point_size, screen_size, multiply;
    SamplerOctree *tree;
    Array<char> *mask;
    bool dtree, dhit, dmesh, dall,
        dblack, dcandidate, dremoved,
        doriginal, dnormals, dxpensive, pointrender;
};

#endif

