#ifndef ZOOM_H
#define ZOOM_H
//=======================================================================
// Zoom class implementation.
// 19/7/2002, Diego Nehab
//=======================================================================

//=======================================================================
// Zoom Class
//=======================================================================
class Zoom {
public:
    //---------------------------------------------------------------------
    // Constructor
    //---------------------------------------------------------------------
    Zoom(double w = 1, double h = 1, double x0 = 0, double y0 = 0);

    //---------------------------------------------------------------------
    // Mouse down event
    // Input
    //     x, y: coordinates
    //---------------------------------------------------------------------
    void Down(double x, double y);

    //---------------------------------------------------------------------
    // Mouse motion event
    // Input
    //     x, y: coordinates
    //---------------------------------------------------------------------
    void Motion(double x, double y);

    //---------------------------------------------------------------------
    // Set scale value
    // Input
    //     a: scale amount
    //---------------------------------------------------------------------
    void Set(const double a = 1);

    //---------------------------------------------------------------------
    // Reset viewport parameters
    //---------------------------------------------------------------------
    void Reset(double w, double h, double x0, double y0);

    //---------------------------------------------------------------------
    // Mouse down event
    // Input
    //     x, y: coordinates
    //---------------------------------------------------------------------
    void Up(double x, double y);

    //---------------------------------------------------------------------
    // Get cumulative zoom
    //---------------------------------------------------------------------
    double GetScale(void) const;

private:

    double gets(double x, double y);

    // are we dragging?
    int drag;
    // cumulative and current scales
    double a, ac;
    // projection and viewport parameters 
    double w, h, x0, y0;
    // mouse down position
    double xp;
};

//=======================================================================
// Public Methods
//=======================================================================
inline 
Zoom::Zoom(double w, double h, double x0, double y0) 
{
    a = 1;
    ac = 1;
    Reset(x0, y0, w, h);
    drag = false;
}

inline void 
Zoom::Reset(double w_, double h_, double x0_, double y0_) 
{
    x0 = x0_;
    y0 = y0_;
    w = w_;
    h = h_;
}

inline void 
Zoom::Down(double x, double y)
{
    (void) y;
    xp = (x-x0)/w;
    drag = true;
    ac = 1;
}

inline void
Zoom::Up(double x, double y)
{
    drag = false;
    a *= gets(x, y); 
    ac = 1;
}

inline void
Zoom::Motion(double x, double y)
{
    if (drag) ac = gets(x, y);
}

inline void
Zoom::Set(const double ua)
{
    a = ua;
}

inline double
Zoom::GetScale(void) const
{
    return a * ac;
}

inline double
Zoom::gets(double x, double y)
{
    (void) y;
    double dp = (x - x0)/w - xp;
    if (dp < 0) return 1.0/(1.0-dp);
    else return 1.0 + dp;
}

#endif // ZOOM_H
